/*
    triseq - Triode sequencer
    Copyright (C) 2008  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include "config.h"


volatile unsigned char cnt1s=100;
volatile unsigned char timer=0;
volatile unsigned char stav=0;

#define START 		0
#define WAIT_UF 	1
#define WAIT_UF_UG	2
#define WAIT_UG 	3
#define WAIT_UG_UAR 4
#define WAIT_UAR    5   
#define WAIT_UAR_UAD 6
#define WAIT_KEY	7
#define READY		8
#define OFF_UA		9
#define WAIT_OFF	10
#define OFF_UG		11



/* 100 Hz */
void timer0(void) interrupt 1{
    TH0 = PTH0; 
    TL0 = PTL0;

    if (cnt1s--==0){
		cnt1s=100;
        /* 1s */
 		if (timer) timer--;     
    }else{
        cnt1s--;
    }
}


void main (void){

    TMOD = 0x21;   		// mode 2 : 8bit TL, preset by TH. // mode 1 : 16 bit
    TH0 = PTH0;
    TL0 = PTL0;
    TCON = 0x11;        // TF1 TR1 TF0 TR0  IE1 IT1 IE0 IT0   
//  SCON = 0x50;        // SM0 SM1 SM2 REN  TB8 RB8 TI  RI    
    IE = 0x82;          // EA  -   -   ES   ET1 EX1 ET0 EX0   

	stav = START;

    while(1){
		/*PIN_KEY_OUT = PIN_UF_IN;
		PIN_UG_OUT = PIN_UG_IN;
		PIN_UAR_OUT = PIN_UA_IN;*/

		switch(stav){
			case START:
				if (!PIN_START){
					PIN_UF_ERR = 0;
					PIN_UG_ERR = 0;
					PIN_UA_ERR = 0;
					while (!PIN_START);
					PIN_UF_ERR = 1;
					PIN_UG_ERR = 1;
					PIN_UA_ERR = 1;
					timer = 5;
                    stav = WAIT_UF;
				}

				if (!PIN_STOP){
					PIN_UF_ERR = 1;
					PIN_UG_ERR = 1;
					PIN_UA_ERR = 1;
				}
				break;
				
			case WAIT_UF:
				if (!PIN_UF_IN){
					timer = 210;              // 210
					stav = WAIT_UF_UG;
				}
				if (!PIN_STOP) stav = START;
				if (!timer){
					PIN_UF_ERR = 0;
					stav = START;
				}
				break;
				
			case WAIT_UF_UG:
				if (!timer){
					PIN_UG_OUT = 0;
					timer = 5;
					stav = WAIT_UG;
				}
				if (!PIN_STOP) stav = START;
				break;
				
			case WAIT_UG:
				if (!PIN_UG_IN){
					timer = 60;			// 60
					stav = WAIT_UG_UAR;
				}
				if (!PIN_STOP) stav = OFF_UG;
				if (!timer){
					PIN_UG_ERR = 0;
					stav = OFF_UG;
				}
				break;
				
			case WAIT_UG_UAR:
				if (!timer){
					PIN_UAR_OUT = 0;
					timer = 5;
					stav = WAIT_UAR;
				}
				if (!PIN_STOP) stav = OFF_UG;
				break;
				
			case WAIT_UAR:
				if (!PIN_UA_IN){
					timer = 5;
					stav = WAIT_UAR_UAD;
				}
				if (!timer){
					PIN_UA_ERR = 0;
					stav = OFF_UA;
				}
				if (!PIN_STOP) stav = OFF_UA;
				break;
				
			case WAIT_UAR_UAD:
				if (!timer){
					PIN_UAD_OUT = 0;
					timer = 2;
					stav = WAIT_KEY;
				}
				if (!PIN_STOP) stav = OFF_UA;
				break;

			case WAIT_KEY:
				if (!timer){
					PIN_KEY_OUT = 0;
					stav = READY;
				}
				if (!PIN_STOP) stav = OFF_UA;
				break;
				
			case READY:
				if (!PIN_STOP) stav = OFF_UA;
				break;
				
			case OFF_UA:
				PIN_KEY_OUT = 1;
				PIN_UAD_OUT = 1;
				PIN_UAR_OUT = 1;
				timer = 10;
				stav = WAIT_OFF;
				break;
				
			case WAIT_OFF:
				if (!timer){
					stav = OFF_UG;
				}
				break;

			case OFF_UG:
				PIN_UG_OUT = 1;
				stav = START;
				break;
		}
    
    }
}
